@@ -26,8 +26,8 @@ def strategy_for_instances(typ):
     return accept_function
 
 class SearchStrategies(SpecificationMapper):
-    def strategy(self, *args, **kwargs):
-        return self.specification_for(*args, **kwargs)
+    def strategy(self, descriptor, **kwargs):
+        return self.specification_for(descriptor, **kwargs)
 
     def missing_specification(self, descriptor):
         if isinstance(descriptor,SearchStrategy):
@@ -56,6 +56,7 @@ class SearchStrategy(object):
                     strategies,
                     descriptor):
         self.descriptor = descriptor
+        strategies.cache_specification_for_descriptor(descriptor, self)
 
     def __repr__(self):
         return "%s(%s)" % (self.__class__.__name__, nice_string(self.descriptor))
@@ -129,7 +130,7 @@ def geometric_probability_for_entropy(desired_entropy):
             return 0.0
     lower = 0.0
     upper = 1.0
-    for _ in xrange(max(int(desired_entropy * 2), 1024)):
+    for _ in xrange(max(int(desired_entropy * 2), 64)):
         mid = (lower + upper) / 2
         if h(mid) > desired_entropy:
             lower = mid
@@ -275,9 +276,15 @@ class TupleStrategy(SearchStrategy):
     def complexity(self, xs):
         return sum((s.complexity(x) for s,x in zip(self.element_strategies, xs)))
 
+    def newtuple(self, xs):
+        if self.descriptor.__class__ == tuple:
+            return tuple(xs)
+        else:
+            return self.descriptor.__class__(*xs)
+
     def produce(self, size, flags):
         es = self.element_strategies
-        return tuple([g.produce(float(size)/len(es),flags) for g in es])
+        return self.newtuple([g.produce(float(size)/len(es),flags) for g in es])
 
     def simplify(self, x):
         """
@@ -291,7 +298,7 @@ class TupleStrategy(SearchStrategy):
             for s in self.element_strategies[i].simplify(x[i]):
                 z = list(x)
                 z[i] = s
-                yield tuple(z)
+                yield self.newtuple(z)
         for i in xrange(0, len(x)):
             for j in xrange(0, len(x)):
                 if i == j: continue
@@ -300,7 +307,7 @@ class TupleStrategy(SearchStrategy):
                         z = list(x)
                         z[i] = s
                         z[j] = t
-                        yield tuple(z)
+                        yield self.newtuple(z)
 
 
 @strategy_for_instances(list)
@@ -516,7 +523,8 @@ class OneOfStrategy(SearchStrategy):
 
     def find_first_strategy(self, x):
         for s in self.element_strategies:
-            if s.could_have_produced(x): return s
+            if s.could_have_produced(x): 
+                return s
         else:
             raise ValueError("Value %s could not have been produced from %s" % (x, self))
 
